package exnihilocreatio.compatibility.jei.sieve;

import com.google.common.collect.HashMultiset;
import com.google.common.collect.Multiset;
import exnihilocreatio.ExNihiloCreatio;
import exnihilocreatio.registries.SieveRegistry;
import exnihilocreatio.registries.types.Siftable;
import exnihilocreatio.util.ItemInfo;
import mezz.jei.api.IGuiHelper;
import mezz.jei.api.gui.IDrawable;
import mezz.jei.api.gui.IDrawableStatic;
import mezz.jei.api.gui.IRecipeLayout;
import mezz.jei.api.gui.ITooltipCallback;
import mezz.jei.api.ingredients.IIngredients;
import mezz.jei.api.recipe.IFocus;
import mezz.jei.api.recipe.IRecipeCategory;
import net.minecraft.client.Minecraft;
import net.minecraft.client.resources.I18n;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ResourceLocation;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

import javax.annotation.Nonnull;
import java.util.List;

public class SieveRecipeCategory implements IRecipeCategory<SieveRecipe> {

    public static final String UID = "exnihilocreatio:sieve";
    private static final ResourceLocation texture = new ResourceLocation(ExNihiloCreatio.MODID, "textures/gui/jei_sieve.png");

    private final IDrawableStatic background;
    private final IDrawableStatic slotHighlight;
    private boolean hasHighlight;
    private int highlightX;
    private int highlightY;

    public SieveRecipeCategory(IGuiHelper helper) {
        this.background = helper.createDrawable(texture, 0, 0, 166, 128);
        this.slotHighlight = helper.createDrawable(texture, 166, 0, 18, 18);
    }

    @Override
    @Nonnull
    public String getUid() {
        return UID;
    }

    @Override
    @Nonnull
    public String getTitle() {
        return "Sieve";
    }

    @Override
    public String getModName() {
        return ExNihiloCreatio.MODID;
    }

    @Override
    @Nonnull
    public IDrawable getBackground() {
        return background;
    }

    @Override
    public void drawExtras(@Nonnull Minecraft minecraft) {
        if (hasHighlight) {
            slotHighlight.draw(minecraft, highlightX, highlightY);
        }
    }

    private void setRecipe(@Nonnull IRecipeLayout recipeLayout, @Nonnull final SieveRecipe recipeWrapper) {
        //Mesh
        recipeLayout.getItemStacks().init(0, true, 61, 9);
        recipeLayout.getItemStacks().set(0, (ItemStack) recipeWrapper.getInputs().get(0));
        //BlockStoneAxle
        recipeLayout.getItemStacks().init(1, true, 87, 9);
        recipeLayout.getItemStacks().set(1, (ItemStack) recipeWrapper.getInputs().get(1));

        IFocus<?> focus = recipeLayout.getFocus();

        if (focus != null) {
            hasHighlight = focus.getMode() == IFocus.Mode.OUTPUT;

            int slotIndex = 2;
            for (int i = 0; i < recipeWrapper.getOutputs().size(); i++) {
                final int slotX = 2 + (i % 9 * 18);
                final int slotY = 36 + (i / 9 * 18);
                ItemStack outputStack = (ItemStack) recipeWrapper.getOutputs().get(i);
                recipeLayout.getItemStacks().init(slotIndex + i, false, slotX, slotY);
                recipeLayout.getItemStacks().set(slotIndex + i, outputStack);

                ItemStack focusStack = (ItemStack) focus.getValue();
                if (focus.getMode() == IFocus.Mode.OUTPUT && !focusStack.func_190926_b()
                        && focusStack.func_77973_b() == outputStack.func_77973_b()
                        && focusStack.func_77952_i() == outputStack.func_77952_i()) {
                    highlightX = slotX;
                    highlightY = slotY;
                }
            }
        }

        recipeLayout.getItemStacks().addTooltipCallback(new ITooltipCallback<ItemStack>() {
            @Override
            @SideOnly(Side.CLIENT)
            public void onTooltip(int slotIndex, boolean input, @Nonnull ItemStack ingredient, @Nonnull List<String> tooltip) {
                if (!input) {
                    ItemStack mesh = (ItemStack) recipeWrapper.getInputs().get(0);
                    Multiset<String> condensedTooltips = HashMultiset.create();
                    for (Siftable siftable : SieveRegistry.getDrops((ItemStack) recipeWrapper.getInputs().get(1))) {
                        if (siftable.getMeshLevel() != mesh.func_77952_i())
                            continue;
                        ItemInfo info = siftable.getDrop();
                        if (info.getItem() != ingredient.func_77973_b() || info.getMeta() != ingredient.func_77952_i())
                            continue;

                        String s;
                        int iChance = (int) (siftable.getChance() * 100f);
                        if (iChance > 0) {
                            s = String.format("%3d%%", (int) (siftable.getChance() * 100f));
                        } else {
                            s = String.format("%1.1f%%", siftable.getChance() * 100f);
                        }
                        condensedTooltips.add(s);
                    }
                    tooltip.add(I18n.func_135052_a("jei.sieve.dropChance"));
                    for (String line : condensedTooltips.elementSet()) {
                        tooltip.add(" * " + condensedTooltips.count(line) + "x " + line);
                    }
                }
            }
        });
    }

    public void setRecipe(@Nonnull IRecipeLayout recipeLayout, @Nonnull SieveRecipe recipeWrapper, @Nonnull IIngredients ingredients) {
        setRecipe(recipeLayout, recipeWrapper); //I'm sure this is bad.

    }

    @Override
    public IDrawable getIcon() {
        return null;
    }
}
