package exnihilocreatio.registries;

import com.google.common.collect.Maps;
import com.google.gson.Gson;
import com.google.gson.GsonBuilder;
import com.google.gson.reflect.TypeToken;
import exnihilocreatio.json.CustomItemInfoJson;
import exnihilocreatio.registries.manager.ExNihiloRegistryManager;
import exnihilocreatio.registries.manager.ICompostDefaultRegistryProvider;
import exnihilocreatio.registries.types.Compostable;
import exnihilocreatio.texturing.Color;
import exnihilocreatio.util.ItemInfo;
import lombok.Getter;
import net.minecraft.block.Block;
import net.minecraft.block.state.IBlockState;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.item.ItemFood;
import net.minecraft.item.ItemStack;
import net.minecraft.util.NonNullList;
import net.minecraftforge.fml.common.FMLCommonHandler;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;
import org.apache.commons.io.IOUtils;

import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.StandardOpenOption;
import java.util.HashMap;
import java.util.Map;

public class CompostRegistry {

    @Getter
    private static Map<ItemInfo, Compostable> registry = new HashMap<>();

    private static Gson gson = new GsonBuilder().setPrettyPrinting().registerTypeAdapter(ItemInfo.class, new CustomItemInfoJson()).create();

    public static void loadJson(File file) {
        registry.clear();

        if (file.exists()) {
            try {
                FileReader fr = new FileReader(file);
                Map<String, Compostable> gsonInput = gson.fromJson(fr, new TypeToken<Map<String, Compostable>>() {
                }.getType());

                for (Map.Entry<String, Compostable> entry : gsonInput.entrySet()) {
                    registry.put(new ItemInfo(entry.getKey()), entry.getValue());
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        } else {
            registerDefaults();
            saveJson(file);
        }
    }

    public static void saveJson(File file) {
        FileWriter fw = null;
        try {
            fw = new FileWriter(file);
            gson.toJson(registry, fw);

        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            IOUtils.closeQuietly(fw);
        }
    }

    public static void register(ItemInfo item, Compostable compostable) {
        registry.put(item, compostable);
    }

    public static void register(Item item, int meta, float value, IBlockState state, Color color) {
        register(new ItemInfo(item, meta), new Compostable(value, color, new ItemInfo(state)));
    }

    public static void register(Block block, int meta, float value, IBlockState state, Color color) {
        register(Item.func_150898_a(block), meta, value, state, color);
    }

    public static Compostable getItem(Item item, int meta) {
        return getItem(new ItemInfo(item, meta));
    }

    public static Compostable getItem(ItemStack stack) {
        return getItem(new ItemInfo(stack));
    }

    public static Compostable getItem(ItemInfo info) {
        return registry.get(info);
    }

    public static boolean containsItem(Item item, int meta) {
        return containsItem(new ItemInfo(item, meta));
    }

    public static boolean containsItem(ItemStack stack) {
        return containsItem(new ItemInfo(stack));
    }

    public static boolean containsItem(ItemInfo info) {
        return registry.containsKey(info);
    }

    public static void registerDefaults() {
        for (ICompostDefaultRegistryProvider provider : ExNihiloRegistryManager.getDefaultCompostRecipeHandlers()) {
            provider.registerCompostRecipeDefaults();
        }
    }

    @SideOnly(Side.CLIENT)
    public static void recommendAllFood(File file) {
        if (FMLCommonHandler.instance().getSide().isServer()) {
            return;
        }

        IBlockState dirt = Blocks.field_150346_d.func_176223_P();
        Color brown = new Color("7F3F0F");

        Map<String, Compostable> recommended = Maps.newHashMap();

        for (Item item : Item.field_150901_e) {
            if (item instanceof ItemFood) {
                ItemFood food = (ItemFood) item;

                NonNullList<ItemStack> stacks = NonNullList.func_191196_a();
                food.func_150895_a(CreativeTabs.field_78039_h, stacks);

                for (ItemStack foodStack : stacks) {
                    ItemInfo foodItemInfo = new ItemInfo(foodStack);

                    if (!containsItem(foodItemInfo)) {
                        int hungerRestored = food.func_150905_g(foodStack);

                        recommended.put(foodItemInfo.toString(), new Compostable(hungerRestored * 0.025F, brown, new ItemInfo(dirt)));
                    }
                }
            }
        }

        String json = gson.toJson(recommended, new TypeToken<Map<String, Compostable>>() {
        }.getType());

        try {
            Files.write(file.toPath(), json.getBytes(), StandardOpenOption.CREATE, StandardOpenOption.TRUNCATE_EXISTING);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }
}
